
(function($) {
	
var StringUtils = (function() {
	return {
		trim: function(str) {
			if (!str) return '';
			return str.replace(/^\s+/, '').replace(/\s+$/, '');
		},
		isEmpty: function(str) {
			if (!str) return true
			return this.trim(str).length == 0;
		},
		entities: function(str) {
			if (!str) return '';
			var entities = [
				['&', '&amp;'],
				['>', '&gt;'],
				['<', '&lt;']
			];
			var n = entities.length;
			for (var i = 0; i < n; ++i) {
				if (str.indexOf(entities[i][0]) != -1) {
					str = str.replace(new RegExp(entities[i][0], 'g'), entities[i][1]);
				}
			}
			return str;
		},
		br: function(str) {
			if (!str) return '';
			if (str.indexOf('\n') != -1 || str.indexOf('\r') != -1) {
				return str.replace(/\r\n|\r|\n/g, '<br />');
			}
			return str;
		},
		textile: function(str) {
			if (!str) return '';
			/*
		 	 * This function originally from Stuart Langridge at http://www.kryogenix.org/
		 	 * Heavily updated by Jeff Minard at http://www.creatimation.net/
		 	 */

			var r = str;			
			if (r.indexOf('*') != -1) {
				re = new RegExp('\\*([^ ][^\\r\\n]*?)\\*', 'g');
				//re = new RegExp('\\*(.+?)\\*', 'g');
				r = r.replace(re, '<strong>$1</strong>');
			}
			if (r.indexOf('_') != -1) {
				re = new RegExp('\\b_(.+?)_\\b', 'g');
				r = r.replace(re, '<em>$1</em>');
			}
			if (r.indexOf('~') != -1) {
				re = new RegExp('~([^ ][^\\r\\n]+?)~', 'g');
				r = r.replace(re, '<sub>$1</sub>');
			}
			if (r.indexOf('^') != -1) {
				re = new RegExp('\\^([^ ][^\\r\\n]+?)\\^', 'g');
				r = r.replace(re, '<sup>$1</sup>');
			}

			// links
			if (r.indexOf('"') != -1) {
				re = new RegExp('"\\b(.+?)\\(\\b(.+?)\\b\\)":([^\\s]+)', 'g');
				r = r.replace(re, '<a href="$3" title="$2">$1</a>');
				re = new RegExp('"\\b(.+?)\\b":([^\\s]+)', 'g');
				r = r.replace(re, '<a href="$2">$1</a>');
			}

			// block level formatting

			// Jeff's hack to show single line breaks as they should.
			// insert breaks - but you get some....stupid ones
			if (r.indexOf('\n') != -1) {
				re = new RegExp('(.*)\n([^#\*\n].*)', 'gi');
				r = r.replace(re, '$1<br />$2');
				// remove the stupid breaks.
				re = new RegExp('\n*<br */>', 'gi');
				r = r.replace(re, '\n');
			}

			var lines = r.split('\n');
			var l = lines.length;
			for (var i = 0; i < l; ++i) {
				var line = lines[i].replace(/\s*$/, '');
				var changed = false;
				if (line.search(/^\s*bq\.\s+/) != -1) { 
					line = line.replace(/^\s*bq\.\s+/,'\t<blockquote>') + '</blockquote>';
					changed = true; 
				}

				if (line.search(/^\s*\*\s+/) != -1) {
					// * for bullet list; make up an liu tag to be fixed later
					line = line.replace(/^\s*\*\s+/, '\t<liu>') + '</liu>';
					changed = true;
				}
				if (line.search(/^\s*#\s+/) != -1) {
					// # for numeric list; make up an lio tag to be fixed later
					line = line.replace(/^\s*#\s+/, '\t<lio>') + '</lio>';
					changed = true;
				}
				if (!changed && (line.replace(/\s/g,'').length > 0)) {
					line = '<p>' + line + '</p>';
				}
				lines[i] = line + '\n';
			}
			// Second pass to do lists
			var inlist = false; 
			var listtype = '';
			for (var i = 0; i < l; ++i) {
				line = lines[i];
				if (inlist && listtype == 'ul' && !line.match(/^\t<liu/)) {
					line = '</ul>\n' + line;
					inlist = false;
				}
				if (inlist && listtype == 'ol' && !line.match(/^\t<lio/)) {
					line = '</ol>\n' + line;
					inlist = false;
				}
				if (!inlist && line.match(/^\t<liu/)) {
					line = '<ul>' + line;
					inlist = true;
					listtype = 'ul';
				}
				if (!inlist && line.match(/^\t<lio/)) {
					line = '<ol>' + line;
					inlist = true;
					listtype = 'ol';
				}
				lines[i] = line;
			}

			r = lines.join('\n');
			// jeff added : will correctly replace <li(o|u)> AND </li(o|u)>
			r = r.replace(/li[o|u]>/g, 'li>');
			return r;
		}
	};
})();


var ExpandableTextarea = function(textarea) {
	var getViewportHeight = function() {
		if (typeof window.innerHeight != 'undefined') {
			return window.innerHeight;
		}
		if (typeof document.documentElement != 'undefined') {
			var de = document.documentElement;
			if (typeof de.clientHeight != 'undefined' && de.clientHeight != 0) {
				return de.clientHeight;
			}
		}
		return document.getElementsByTagName('body')[0].clientHeight;
	};
	
	var handlePageResize = function() {
		availableHeight = Math.max(getViewportHeight() - 36, 36);
	};
	
	var resizeTextarea = function() {
		if (arguments.callee.text != this.value) {
			dummy.html(StringUtils.br(StringUtils.entities(this.value)));
			var h = dummy.height();
			var targetHeight = (h > (height - 36)) ? h + 36 : height;
			if (targetHeight < availableHeight) {
				textarea.css({height: targetHeight + 'px'});
			}
			arguments.callee.text = this.value;
		}
	};
	
	// create a dummy element used to calculate textarea size
	var id = (textarea.attr('id') || new Date().getTime()) + '-dummy';
	$('body').append('<div id="' + id + '"></div>');
	
	var dummy = $('#' + id);
	var width = textarea.width();
	var height = textarea.height();
	var availableHeight = 36;
	// IE can go weird here, font-size and font-family need to be in the stylesheet
	var fontSize = textarea.css('font-size');
	if (parseInt(fontSize) > 100) { // sanity check on the font size
		fontSize = '1em';
	}
	var fontFamily = textarea.css('font-family');
	var fontWeight = textarea.css('font-weight');
	var lineHeight = textarea.css('line-height');

	dummy.css({
		visibility: 'hidden',
		position: 'absolute', 
		left: 0, 
		top: 0,
		width: width + 'px', 
		fontSize: fontSize, 
		fontFamily: fontFamily, 
		fontWeight: fontWeight,
		lineHeight: lineHeight
	});
	
	var updateInterval = false;
	textarea.bind('focus', function() {
		var self = this;
		updateInterval = setInterval(function() {
			resizeTextarea.apply(self);
		}, 250);
	}).bind('blur', function() {
		clearInterval(updateInterval);
		resizeTextarea.apply(this);
	});
	
	handlePageResize();
	$(document).bind('resize', handlePageResize);
	textarea.trigger('keyup');
};

$(function() {
	var oldIE = $.browser.msie && $.browser.version < 7;
	var commentField = $('#comment-body');
	var commentPreview = $('#comment-preview');
	var commentPreviewContent = $('#comment-preview dl.ugc > dd');
	var commentPreviewDate = $('#comment-preview dl.date > dd');
	var updateInterval = false;
	var scope = null;
		
	var doPreview = function() {
		if (arguments.callee.text !== this.value) {
			if (!StringUtils.isEmpty(this.value)) {
				commentPreviewContent.html(StringUtils.textile(StringUtils.entities(this.value)));
				commentPreview.css({display: 'block'});
			} else {
				commentPreview.css({display: 'none'});
			}
			arguments.callee.text = this.value;
		}
	};
	
	if (oldIE) { // before IE 7, IE has issue with the page skipping when content is inserted - particularly on long pages
		$('#comment-preview-button').bind('click', function() {
			doPreview.apply(commentField[0]);
			setTimeout(function() {
				window.scrollTo(0, commentPreview.offset().top);
			}, 500);
			return false;
		});
	} else {
		commentField.bind('focus', function() {
			scope = this;	
			updateInterval = setInterval(function() {
				doPreview.apply(scope);
			}, 250);
		}).bind('blur', function() {
			clearInterval(updateInterval);
			doPreview.apply(scope);
			scope = null;
		});
		$('#comment-preview-button').css({display: 'none'});
	}
	$('#comment-help-text, #comment-preview > h2').css({display: 'none'});
	
	var instructionText = 'How do I enter formatted text?';
	$('#comment-help').append('<p><a href="javascript:;" id="comment-help-link">' + instructionText + '</a></p>');
	$('#comment-help-link').bind('click', function() {
		if (!arguments.callee.expanded) {
			$('#comment-help-text').slideDown('fast');
			$(this).text('Close');
		} else {
			$('#comment-help-text').slideUp('fast');
			$(this).text(instructionText);
		}
		arguments.callee.expanded = !arguments.callee.expanded;
	});
	
	var togglePostedBy = function() {
		if (this.checked) {
			$('#comment-preview .ugc dt').html(postedAnon);
		} else {
			$('#comment-preview .ugc dt').html(postedBy);
		}
	};
	var postedAnon = '<cite class="vcard">This comment will be made anonymously.</cite>';
	var postedBy = $('#comment-preview .ugc dt').html();
	$('#comment-anon').bind('click', togglePostedBy);
	togglePostedBy.apply($('#comment-anon').get(0));
	
	if (!oldIE) {
		var textarea = new ExpandableTextarea(commentField);
	}
	doPreview.apply(commentField[0]);
});
	
})(jQuery);
